from whatsapp_api_client_python.response import Response


class Receiving:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def downloadFile(self, idMessage: str) -> Response:
        'The method is aimed for downloading incoming and outgoing files. '\
        'Links to incoming files are transmitted in Incoming messages, '\
        'and you can also get them using LastIncomingMessages method. '\
        'You can get links to outgoing files using '\
        'LastOutgoingMessages method.'\
        'Files storage period and, accordingly, the capability '\
        'to download them is limited to 24 hours.'

        requestBody = {
            'idMessage': idMessage,
        }

        return self.restApi.request('POST', 
            '{{host}}/waInstance{{idInstance}}'
            '/DownloadFile/{{apiTokenInstance}}',
            requestBody)

    def receiveNotification(self) -> Response:
        'The method is aimed for receiving one incoming notification from '\
        'the notifications queue. '\
        'ReceiveNotification method waits for a notification receipt '\
        'for 20 sec. The method call ends with an empty response if a timeout '\
        'is reached. If a notification comes to the queue within 20 seconds, '\
        'the method call is completed, and the method returns the received '\
        'notification.'\
        'After receiving and processing an incoming notification, you need '\
        'to delete the notification from the queue. This requires you to run '\
        'DeleteNotification method. After calling DeleteNotification method, '\
        'the notification will be considered received and processed and '\
        'will be permanently deleted from the queue. Therefore, the next call '\
        'of ReceiveNotification method will return the next notification from '\
        'the queue in the order in which notifications come to the queue.'\
        'Incoming notifications are stored in the queue for 24 hours. '\
        'Notifications are sent from the queue in FIFO order'

        return self.restApi.request('GET', 
            '{{host}}/waInstance{{idInstance}}'
            '/ReceiveNotification/{{apiTokenInstance}}')

    def deleteNotification(self, receiptId: int) -> Response:
        'The method is aimed for deleting an incoming notification from '\
        'the notification queue. To specify what notification to delete, '\
        'use receiptId parameter. After receiving and processing an incoming '\
        'notification, you need to delete the notification from the queue. '\
        'This requires you to run this method. After calling the method, '\
        'the notification will be considered received and processed and '\
        'will be permanently deleted from the queue. Therefore, the next call '\
        'of ReceiveNotification method will return the next notification from '\
        'the queue in the order in which notifications come to the queue.'\
        'Incoming notifications are stored in the queue for 24 hours.'\
        'Notifications are sent from the queue in FIFO order'

        return self.restApi.request('DELETE', 
            '{{host}}/waInstance{{idInstance}}'
            '/DeleteNotification/{{apiTokenInstance}}/' + str(receiptId))


from array import array
import requests
import json
from whatsapp_api_client_python.response import Response

from whatsapp_api_client_python.tools.account import Account
from whatsapp_api_client_python.tools.device import Device
from whatsapp_api_client_python.tools.groups import Groups
from whatsapp_api_client_python.tools.journals import Journals
from whatsapp_api_client_python.tools.marking import Marking
from whatsapp_api_client_python.tools.queues import Queues
from whatsapp_api_client_python.tools.receiving import Receiving
from whatsapp_api_client_python.tools.sending import Sending
from whatsapp_api_client_python.tools.serviceMethods import ServiceMethods

class RestApi:
    'REST API class'

    host: str
    idInstance: str
    apiTokenInstance: str

    def __init__(self, host: str, 
                    idInstance: str, 
                    apiTokenInstance: str) -> None:
        self.host = host
        self.idInstance = idInstance
        self.apiTokenInstance = apiTokenInstance

        self.account = Account(self)
        self.device = Device(self)
        self.groups = Groups(self)
        self.journals = Journals(self)
        self.marking = Marking(self)
        self.queues = Queues(self)
        self.receiving = Receiving(self)
        self.sending = Sending(self)
        self.serviceMethods = ServiceMethods(self)

    def request(self, method: str, url: str, 
                payload: any = None, files: array = None):
        url = url.replace('{{host}}', self.host)
        url = url.replace('{{idInstance}}', self.idInstance)
        url = url.replace('{{apiTokenInstance}}', self.apiTokenInstance)
        status_code = 0
        text = ''
        try:
            headers = {}
            payloadJson = None
            if payload != None:
                headers = {
                    'Content-Type': 'application/json'
                }
                payloadJson = json.dumps(payload)
            result = requests.request(method, url, headers = headers, 
                                        data = payloadJson, 
                                        files = files)
            result.raise_for_status()
            status_code = result.status_code
            text = result.text
        except requests.HTTPError as http_err:
            status_code = 0
            text = f'HTTP {method} error occurred: {http_err}'
        except Exception as err:
            status_code = 0
            text = f'Other error occurred: {err}'
        return Response(status_code, text)


from whatsapp_api_client_python.response import Response


class Marking:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def readChat(self, chatId: str, idMessage: str) -> Response:
            'The method returns the chat message history.'

            requestBody = {
                'chatId': chatId,
                'idMessage': idMessage,
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/ReadChat/{{apiTokenInstance}}',
                requestBody)


from whatsapp_api_client_python.response import Response


class ServiceMethods:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def checkWhatsapp(self, phoneNumber: int) -> Response:
            'The method checks WhatsApp account availability on a phone number.'

            requestBody = {
                'phoneNumber': phoneNumber
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/CheckWhatsapp/{{apiTokenInstance}}',
                requestBody)

    def getAvatar(self, chatId: str) -> Response:
            'The method returns a user or a group chat avatar.'

            requestBody = {
                'chatId': chatId
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetAvatar/{{apiTokenInstance}}',
                requestBody)

    def getContactInfo(self, chatId: str) -> Response:
            'The method is aimed for getting information on a contact.'

            requestBody = {
                'chatId': chatId
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetContactInfo/{{apiTokenInstance}}',
                requestBody)

    def getContacts(self, chatId: str) -> Response:
            'The method is aimed for getting a list of the current account '\
            'contacts. Sends contacts of all recipients whom the account '\
            'connected with. Parameter "contact name" "name" takes on '\
            'a value based on the below criteria: '\
            'If the account is recorded in the phonebook, then we get the '\
            'name from the book; '\
            'If the account is not recorded in the phonebook, then we get '\
            'the name from WhatsApp account;'\
            'If the account is not recorded in the phone book and WhatsApp '\
            'account name is not assigned, then we get an empty field.'

            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetContacts/{{apiTokenInstance}}')

    def archiveChat(self, chatId: str) -> Response:
            'The method archives a chat. You can archive chats that have at '\
            'least one incoming message.'

            requestBody = {
                'chatId': chatId
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/ArchiveChat/{{apiTokenInstance}}',
                requestBody)

    def deleteMessage(self, chatId: str, idMessage: str) -> Response:
            'The method deletes a message from a chat.'

            requestBody = {
                'chatId': chatId,
                'idMessage': idMessage
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/DeleteMessage/{{apiTokenInstance}}',
                requestBody)

    def unarchiveChat(self, chatId: str) -> Response:
            'The method deletes a message from a chat.'

            requestBody = {
                'chatId': chatId
            }

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/UnarchiveChat/{{apiTokenInstance}}',
                requestBody)


from whatsapp_api_client_python.response import Response


class Queues:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def clearMessagesQueue(self) -> Response:
            'The method is aimed for clearing the queue of messages to be sent.'

            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/ClearMessagesQueue/{{apiTokenInstance}}')

    def showMessagesQueue(self) -> Response:
            'The method is aimed for getting a list of messages in the queue '\
            'to be sent. Messages sending rate is managed by Messages sending '\
            'delay parameter.'

            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/ShowMessagesQueue/{{apiTokenInstance}}')


import json


class Response:
    code: int
    data: json
    error: str

    def __init__(self, code: int, text: str) -> None:
        self.code = code
        if code == 200:
            self.data = json.loads(text)
        else:
            self.error = text


from whatsapp_api_client_python.response import Response


class Device:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def getDeviceInfo(self) -> Response:
            'The method is aimed for getting information about the device '\
            '(phone) running WhatsApp Business application.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/GetDeviceInfo/{{apiTokenInstance}}')


from array import array
import os.path
from whatsapp_api_client_python.response import Response


class Sending:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def sendButtons(self, chatId: str, message: str, footer: str,
                    buttons: array, quotedMessageId: str = None, 
                    archiveChat: bool = None) -> Response:
            'The method is aimed for sending a button message to a personal '\
            'or a group chat. The message will be added to the send queue. '\
            'Checking whatsapp authorization on the phone (i.e. availability '\
            'in linked devices) is not performed. The message will be kept '\
            'for 24 hours in the queue and will be sent immediately after '\
            'phone authorization. The rate at which messages are sent from '\
            'the queue is managed by Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'message': message,
                'footer': footer,
                'buttons': buttons,
            }

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId
            if archiveChat != None:
                requestBody['archiveChat'] = archiveChat

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendButtons/{{apiTokenInstance}}',
                requestBody)

    def sendContact(self, chatId: str, contact: object,
                    quotedMessageId: str = None) -> Response:
            'The method is aimed for sending a contact message. '\
            'Contact visit card is created and sent to a chat. '\
            'The message will be added to the send queue. '\
            'Linked device not required when sending. '\
            'Messages will be kept for 24 hours in the queue until account '\
            'will be authorized The rate at which messages are sent from '\
            'the queue is managed by Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'contact': contact
            }

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendContact/{{apiTokenInstance}}',
                requestBody)

    def sendFileByUpload(self, chatId: str, path: str,
                    fileName: str = None,
                    caption: str = None,
                    quotedMessageId: str = None) -> Response:
            'The method is aimed for sending a file uploaded by form '\
            '(form-data). The message will be added to the send queue. '\
            'The rate at which messages are sent from the queue is managed '\
            'by Message sending delay parameter.'\
            'Video, audio and image files available for viewing and listening '\
            'to are sent as in native-mode WhatsApp. Documents are sent in '\
            'the same way as in native-mode WhatsApp. Outgoing file type and '\
            'send method is determined by the file extension. '\
            'Description is only added to images and video.'\
            'The maximum size of outgoing files is 37 MB.'

            pathParts = os.path.split(path)
            if fileName == None:
                fileName = pathParts[1]

            files = [
                ('file', open(path,'rb'))
            ]

            requestBody = {
                'chatId': chatId,
                'fileName': fileName
            }

            if caption != None:
                requestBody['caption'] = caption

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendFileByUpload/{{apiTokenInstance}}',
                requestBody, files)

    def sendFileByUrl(self, chatId: str, urlFile: str,
                    fileName: str = None,
                    caption: str = None,
                    quotedMessageId: str = None,
                    archiveChat: bool = None) -> Response:
            'The method is aimed for sending a file uploaded by Url '\
            'The message will be added to the send queue. '\
            'The rate at which messages are sent from the queue is managed '\
            'by Message sending delay parameter.'\
            'Video, audio and image files available for viewing and listening '\
            'to are sent as in native-mode WhatsApp. Documents are sent in '\
            'the same way as in native-mode WhatsApp. Outgoing file type and '\
            'send method is determined by the file extension. '\
            'Description is only added to images and video.'\
            'The maximum size of outgoing files is 37 MB.'

            requestBody = {
                'chatId': chatId,
                'urlFile': urlFile,
                'fileName': fileName
            }

            if caption != None:
                requestBody['caption'] = caption

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId

            if archiveChat != None:
                requestBody['archiveChat'] = archiveChat

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendFileByUrl/{{apiTokenInstance}}',
                requestBody)

    def sendLink(self, chatId: str, urlLink: str,
                    quotedMessageId: str = None) -> Response:
            'The method is aimed for sending a message with a link, by which '\
            'an image preview, title and description will be added. '\
            'Linked device not required when sending. Messages will be kept '\
            'for 24 hours in the queue until account will be authorized '\
            'Image, title and description are obtained from Open Graph page '\
            'template being linked to. The message will be added to the send '\
            'queue. The rate at which messages are sent from the queue is '\
            'managed by Messages sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'urlLink': urlLink
            }

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendLink/{{apiTokenInstance}}',
                requestBody)

    def sendListMessage(self, chatId: str, message: str, sections: array,
                    title: str = None,
                    footer: str = None,
                    buttonText: str = None,
                    quotedMessageId: str = None,
                    archiveChat: str = None) -> Response:
            'The method is aimed for sending a message with a select button '\
            'from a list of values to a personal or a group chat. '\
            'The message will be added to the send queue. Checking whatsapp '\
            'authorization on the phone (i.e. availability in linked devices) '\
            'is not performed. The message will be kept for 24 hours in the '\
            'queue and will be sent immediately after phone authorization. '\
            'The rate at which messages are sent from the queue is managed by '\
            'Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'message': message,
                'sections': sections
            }

            if title != None:
                requestBody['title'] = title
            if footer != None:
                requestBody['footer'] = footer
            if buttonText != None:
                requestBody['buttonText'] = buttonText
            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId 
            if archiveChat != None:
                requestBody['archiveChat'] = archiveChat

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendListMessage/{{apiTokenInstance}}',
                requestBody)

    def sendLocation(self, chatId: str, latitude: float, longitude: float,
                    nameLocation: str = None,
                    address: str = None,
                    quotedMessageId: str = None) -> Response:
            'The method is aimed for sending location message. The message '\
            'will be added to the send queue. Linked device not required '\
            'when sending. Messages will be kept for 24 hours in the queue '\
            'until account will be authorized The rate at which messages are '\
            'sent from the queue is managed by Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'latitude': latitude,
                'longitude': longitude
            }

            if nameLocation != None:
                requestBody['nameLocation'] = nameLocation
            if address != None:
                requestBody['address'] = address
            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId 

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendLocation/{{apiTokenInstance}}',
                requestBody)

    def sendMessage(self, chatId: str, message: str,
                    quotedMessageId: str = None,
                    archiveChat: str = None) -> Response:
            'The method is aimed for sending a text message to a personal or '\
            'a group chat. The message will be added to the send queue. '\
            'Linked device not required when sending. Messages will be kept '\
            'for 24 hours in the queue until account will be authorized '\
            'The rate at which messages are sent from the queue is managed '\
            'by Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'message': message
            }

            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId
            if archiveChat != None:
                requestBody['archiveChat'] = archiveChat 

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendMessage/{{apiTokenInstance}}',
                requestBody)

    def sendTemplateButtons(self, chatId: str, message: str,
                    templateButtons: array,
                    footer: str = None,
                    quotedMessageId: str = None,
                    archiveChat: str = None) -> Response:
            'The method is aimed for sending a message with template list '\
            'interacrive buttons to a personal or a group chat. The message '\
            'will be added to the send queue. Checking whatsapp authorization '\
            'on the phone (i.e. availability in linked devices) '\
            'is not performed. The message will be kept for 24 hours in the '\
            'queue and will be sent immediately after phone authorization. '\
            'The rate at which messages are sent from the queue is managed '\
            'by Message sending delay parameter.'

            requestBody = {
                'chatId': chatId,
                'message': message,
                'templateButtons': templateButtons
            }

            if footer != None:
                requestBody['footer'] = footer
            if quotedMessageId != None:
                requestBody['quotedMessageId'] = quotedMessageId
            if archiveChat != None:
                requestBody['archiveChat'] = archiveChat 

            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SendTemplateButtons/{{apiTokenInstance}}',
                requestBody)


import os.path
from whatsapp_api_client_python.response import Response


class Account:
    def __init__(self, restApi) -> None:
        self.restApi = restApi
        
    def getSettings(self) -> Response:
            'The method is aimed for getting the '\
            'current account settings.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/getSettings/{{apiTokenInstance}}')
            
    def getStateInstance(self) -> Response:
            'The method is aimed for getting the account state.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/getStateInstance/{{apiTokenInstance}}')

    def getStatusInstance(self) -> Response:
            'The method is aimed for getting the status of the account instance '\
            'socket connection with WhatsApp.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/getStatusInstance/{{apiTokenInstance}}')

    def logout(self) -> Response:
            'The method is aimed for logging out an account.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/Logout/{{apiTokenInstance}}')

    def qr(self) -> Response:
            'The method is aimed for getting QR code. To authorize your account, '\
            'you have to scan a QR code from application WhatsApp Business'\
            'on your phone. You can also get a QR code and authorize your'\
            'account in your profile.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/QR/{{apiTokenInstance}}')

    def reboot(self) -> Response:
            'The method is aimed for rebooting an account.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/Reboot/{{apiTokenInstance}}')

    def setProfilePicture(self, path) -> Response:
            'The method is aimed for setting an account picture.'

            pathParts = os.path.split(path)
            file = pathParts[1]

            files = [
                ('file',(file, open(path,'rb'),'image/jpeg'))
            ]
            
            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SetProfilePicture/{{apiTokenInstance}}', None, files)

    def setSettings(self, requestBody) -> Response:
            'The method is aimed for setting account settings. '\
            'When this method is requested, the account is rebooted.'
            
            return self.restApi.request('POST', 
                '{{host}}/waInstance{{idInstance}}'
                '/SetSettings/{{apiTokenInstance}}',
                requestBody)

    def setSettings(self) -> Response:
            'The method is aimed for setting a system proxy. '\
            'Use the method when you need to reset custom proxy '\
            'settings to system ones.'
            
            return self.restApi.request('GET', 
                '{{host}}/waInstance{{idInstance}}'
                '/SetSystemProxy/{{apiTokenInstance}}')